@@ -3,33 +3,72 @@
 # License: BSD 3 clause
 
 import numpy as np
+from numpy.testing import assert_allclose
 import pytest
 from scipy import interpolate, sparse
 from copy import deepcopy
+import joblib
+from distutils.version import LooseVersion
 
 from sklearn.datasets import load_boston
+from sklearn.datasets import make_regression
 from sklearn.exceptions import ConvergenceWarning
-from sklearn.utils.testing import assert_array_almost_equal
-from sklearn.utils.testing import assert_almost_equal
-from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import assert_true
-from sklearn.utils.testing import assert_greater
-from sklearn.utils.testing import assert_raises
-from sklearn.utils.testing import assert_raises_regex
-from sklearn.utils.testing import assert_raise_message
-from sklearn.utils.testing import assert_warns
-from sklearn.utils.testing import assert_warns_message
-from sklearn.utils.testing import ignore_warnings
-from sklearn.utils.testing import assert_array_equal
-from sklearn.utils.testing import TempMemmap
+from sklearn.utils._testing import assert_array_almost_equal
+from sklearn.utils._testing import assert_almost_equal
+from sklearn.utils._testing import assert_raises
+from sklearn.utils._testing import assert_raises_regex
+from sklearn.utils._testing import assert_raise_message
+from sklearn.utils._testing import assert_warns
+from sklearn.utils._testing import assert_warns_message
+from sklearn.utils._testing import ignore_warnings
+from sklearn.utils._testing import assert_array_equal
+from sklearn.utils._testing import TempMemmap
 
-from sklearn.linear_model.coordinate_descent import Lasso, \
+from sklearn.linear_model import Lasso, \
     LassoCV, ElasticNet, ElasticNetCV, MultiTaskLasso, MultiTaskElasticNet, \
     MultiTaskElasticNetCV, MultiTaskLassoCV, lasso_path, enet_path
 from sklearn.linear_model import LassoLarsCV, lars_path
+from sklearn.linear_model._coordinate_descent import _set_order
 from sklearn.utils import check_array
 
 
+@pytest.mark.parametrize('order', ['C', 'F'])
+@pytest.mark.parametrize('input_order', ['C', 'F'])
+def test_set_order_dense(order, input_order):
+    """Check that _set_order returns arrays with promised order."""
+    X = np.array([[0], [0], [0]], order=input_order)
+    y = np.array([0, 0, 0], order=input_order)
+    X2, y2 = _set_order(X, y, order=order)
+    if order == 'C':
+        assert X2.flags['C_CONTIGUOUS']
+        assert y2.flags['C_CONTIGUOUS']
+    elif order == 'F':
+        assert X2.flags['F_CONTIGUOUS']
+        assert y2.flags['F_CONTIGUOUS']
+
+    if order == input_order:
+        assert X is X2
+        assert y is y2
+
+
+@pytest.mark.parametrize('order', ['C', 'F'])
+@pytest.mark.parametrize('input_order', ['C', 'F'])
+def test_set_order_sparse(order, input_order):
+    """Check that _set_order returns sparse matrices in promised format."""
+    X = sparse.coo_matrix(np.array([[0], [0], [0]]))
+    y = sparse.coo_matrix(np.array([0, 0, 0]))
+    sparse_format = "csc" if input_order == "F" else "csr"
+    X = X.asformat(sparse_format)
+    y = X.asformat(sparse_format)
+    X2, y2 = _set_order(X, y, order=order)
+    if order == 'C':
+        assert sparse.isspmatrix_csr(X2)
+        assert sparse.isspmatrix_csr(y2)
+    elif order == 'F':
+        assert sparse.isspmatrix_csc(X2)
+        assert sparse.isspmatrix_csc(y2)
+
+
 def test_lasso_zero():
     # Check that the lasso can handle zero data without crashing
     X = [[0], [0], [0]]
@@ -146,38 +185,37 @@ def build_dataset(n_samples=50, n_features=200, n_informative_features=10,
     return X, y, X_test, y_test
 
 
-@pytest.mark.filterwarnings('ignore: You should specify a value')  # 0.22
 def test_lasso_cv():
     X, y, X_test, y_test = build_dataset()
     max_iter = 150
-    clf = LassoCV(n_alphas=10, eps=1e-3, max_iter=max_iter).fit(X, y)
+    clf = LassoCV(n_alphas=10, eps=1e-3, max_iter=max_iter, cv=3).fit(X, y)
     assert_almost_equal(clf.alpha_, 0.056, 2)
 
-    clf = LassoCV(n_alphas=10, eps=1e-3, max_iter=max_iter, precompute=True)
+    clf = LassoCV(n_alphas=10, eps=1e-3, max_iter=max_iter, precompute=True,
+                  cv=3)
     clf.fit(X, y)
     assert_almost_equal(clf.alpha_, 0.056, 2)
 
     # Check that the lars and the coordinate descent implementation
     # select a similar alpha
-    lars = LassoLarsCV(normalize=False, max_iter=30).fit(X, y)
+    lars = LassoLarsCV(normalize=False, max_iter=30, cv=3).fit(X, y)
     # for this we check that they don't fall in the grid of
     # clf.alphas further than 1
-    assert_true(np.abs(
-        np.searchsorted(clf.alphas_[::-1], lars.alpha_) -
-        np.searchsorted(clf.alphas_[::-1], clf.alpha_)) <= 1)
+    assert np.abs(np.searchsorted(clf.alphas_[::-1], lars.alpha_) -
+                  np.searchsorted(clf.alphas_[::-1], clf.alpha_)) <= 1
     # check that they also give a similar MSE
     mse_lars = interpolate.interp1d(lars.cv_alphas_, lars.mse_path_.T)
     np.testing.assert_approx_equal(mse_lars(clf.alphas_[5]).mean(),
                                    clf.mse_path_[5].mean(), significant=2)
 
     # test set
-    assert_greater(clf.score(X_test, y_test), 0.99)
+    assert clf.score(X_test, y_test) > 0.99
 
 
 def test_lasso_cv_with_some_model_selection():
     from sklearn.pipeline import make_pipeline
     from sklearn.preprocessing import StandardScaler
-    from sklearn.model_selection import StratifiedKFold
+    from sklearn.model_selection import ShuffleSplit
     from sklearn import datasets
     from sklearn.linear_model import LassoCV
 
@@ -187,7 +225,7 @@ def test_lasso_cv_with_some_model_selection():
 
     pipe = make_pipeline(
         StandardScaler(),
-        LassoCV(cv=StratifiedKFold(n_splits=5))
+        LassoCV(cv=ShuffleSplit(random_state=0))
     )
     pipe.fit(X, y)
 
@@ -233,7 +271,6 @@ def test_lasso_path_return_models_vs_new_return_gives_same_coefficients():
         decimal=1)
 
 
-@pytest.mark.filterwarnings('ignore: You should specify a value')  # 0.22
 def test_enet_path():
     # We use a large number of samples and of informative features so that
     # the l1_ratio selected is more toward ridge than lasso
@@ -252,7 +289,7 @@ def test_enet_path():
     assert_almost_equal(clf.alpha_, min(clf.alphas_))
     # Non-sparse ground truth: we should have selected an elastic-net
     # that is closer to ridge than to lasso
-    assert_equal(clf.l1_ratio_, min(clf.l1_ratio))
+    assert clf.l1_ratio_ == min(clf.l1_ratio)
 
     clf = ElasticNetCV(alphas=[0.01, 0.05, 0.1], eps=2e-3,
                        l1_ratio=[0.5, 0.7], cv=3,
@@ -264,11 +301,11 @@ def test_enet_path():
     assert_almost_equal(clf.alpha_, min(clf.alphas_))
     # Non-sparse ground truth: we should have selected an elastic-net
     # that is closer to ridge than to lasso
-    assert_equal(clf.l1_ratio_, min(clf.l1_ratio))
+    assert clf.l1_ratio_ == min(clf.l1_ratio)
 
     # We are in well-conditioned settings with low noise: we should
     # have a good test-set performance
-    assert_greater(clf.score(X_test, y_test), 0.99)
+    assert clf.score(X_test, y_test) > 0.99
 
     # Multi-output/target case
     X, y, X_test, y_test = build_dataset(n_features=10, n_targets=3)
@@ -277,8 +314,8 @@ def test_enet_path():
     ignore_warnings(clf.fit)(X, y)
     # We are in well-conditioned settings with low noise: we should
     # have a good test-set performance
-    assert_greater(clf.score(X_test, y_test), 0.99)
-    assert_equal(clf.coef_.shape, (3, 10))
+    assert clf.score(X_test, y_test) > 0.99
+    assert clf.coef_.shape == (3, 10)
 
     # Mono-output should have same cross-validated alpha_ and l1_ratio_
     # in both cases.
@@ -291,7 +328,6 @@ def test_enet_path():
     assert_almost_equal(clf1.alpha_, clf2.alpha_)
 
 
-@pytest.mark.filterwarnings('ignore: You should specify a value')  # 0.22
 def test_path_parameters():
     X, y, _, _ = build_dataset()
     max_iter = 100
@@ -300,8 +336,8 @@ def test_path_parameters():
                        l1_ratio=0.5, tol=1e-3)
     clf.fit(X, y)  # new params
     assert_almost_equal(0.5, clf.l1_ratio)
-    assert_equal(50, clf.n_alphas)
-    assert_equal(50, len(clf.alphas_))
+    assert 50 == clf.n_alphas
+    assert 50 == len(clf.alphas_)
 
 
 def test_warm_start():
@@ -363,12 +399,11 @@ def test_enet_cv_positive_constraint():
     assert min(enetcv_constrained.coef_) >= 0
 
 
-@pytest.mark.filterwarnings('ignore: You should specify a value')  # 0.22
 def test_uniform_targets():
-    enet = ElasticNetCV(fit_intercept=True, n_alphas=3)
-    m_enet = MultiTaskElasticNetCV(fit_intercept=True, n_alphas=3)
-    lasso = LassoCV(fit_intercept=True, n_alphas=3)
-    m_lasso = MultiTaskLassoCV(fit_intercept=True, n_alphas=3)
+    enet = ElasticNetCV(n_alphas=3)
+    m_enet = MultiTaskElasticNetCV(n_alphas=3)
+    lasso = LassoCV(n_alphas=3)
+    m_lasso = MultiTaskLassoCV(n_alphas=3)
 
     models_single_task = (enet, lasso)
     models_multi_task = (m_enet, m_lasso)
@@ -438,7 +473,7 @@ def test_enet_multitarget():
     n_targets = 3
     X, y, _, _ = build_dataset(n_samples=10, n_features=8,
                                n_informative_features=10, n_targets=n_targets)
-    estimator = ElasticNet(alpha=0.01, fit_intercept=True)
+    estimator = ElasticNet(alpha=0.01)
     estimator.fit(X, y)
     coef, intercept, dual_gap = (estimator.coef_, estimator.intercept_,
                                  estimator.dual_gap_)
@@ -458,34 +493,32 @@ def test_multioutput_enetcv_error():
     assert_raises(ValueError, clf.fit, X, y)
 
 
-@pytest.mark.filterwarnings('ignore: You should specify a value')  # 0.22
 def test_multitask_enet_and_lasso_cv():
     X, y, _, _ = build_dataset(n_features=50, n_targets=3)
-    clf = MultiTaskElasticNetCV().fit(X, y)
+    clf = MultiTaskElasticNetCV(cv=3).fit(X, y)
     assert_almost_equal(clf.alpha_, 0.00556, 3)
-    clf = MultiTaskLassoCV().fit(X, y)
+    clf = MultiTaskLassoCV(cv=3).fit(X, y)
     assert_almost_equal(clf.alpha_, 0.00278, 3)
 
     X, y, _, _ = build_dataset(n_targets=3)
     clf = MultiTaskElasticNetCV(n_alphas=10, eps=1e-3, max_iter=100,
-                                l1_ratio=[0.3, 0.5], tol=1e-3)
+                                l1_ratio=[0.3, 0.5], tol=1e-3, cv=3)
     clf.fit(X, y)
-    assert_equal(0.5, clf.l1_ratio_)
-    assert_equal((3, X.shape[1]), clf.coef_.shape)
-    assert_equal((3, ), clf.intercept_.shape)
-    assert_equal((2, 10, 3), clf.mse_path_.shape)
-    assert_equal((2, 10), clf.alphas_.shape)
+    assert 0.5 == clf.l1_ratio_
+    assert (3, X.shape[1]) == clf.coef_.shape
+    assert (3, ) == clf.intercept_.shape
+    assert (2, 10, 3) == clf.mse_path_.shape
+    assert (2, 10) == clf.alphas_.shape
 
     X, y, _, _ = build_dataset(n_targets=3)
-    clf = MultiTaskLassoCV(n_alphas=10, eps=1e-3, max_iter=100, tol=1e-3)
+    clf = MultiTaskLassoCV(n_alphas=10, eps=1e-3, max_iter=100, tol=1e-3, cv=3)
     clf.fit(X, y)
-    assert_equal((3, X.shape[1]), clf.coef_.shape)
-    assert_equal((3, ), clf.intercept_.shape)
-    assert_equal((10, 3), clf.mse_path_.shape)
-    assert_equal(10, len(clf.alphas_))
+    assert (3, X.shape[1]) == clf.coef_.shape
+    assert (3, ) == clf.intercept_.shape
+    assert (10, 3) == clf.mse_path_.shape
+    assert 10 == len(clf.alphas_)
 
 
-@pytest.mark.filterwarnings('ignore: You should specify a value')  # 0.22
 def test_1d_multioutput_enet_and_multitask_enet_cv():
     X, y, _, _ = build_dataset(n_features=10)
     y = y[:, np.newaxis]
@@ -499,7 +532,6 @@ def test_1d_multioutput_enet_and_multitask_enet_cv():
     assert_almost_equal(clf.intercept_, clf1.intercept_[0])
 
 
-@pytest.mark.filterwarnings('ignore: You should specify a value')  # 0.22
 def test_1d_multioutput_lasso_and_multitask_lasso_cv():
     X, y, _, _ = build_dataset(n_features=10)
     y = y[:, np.newaxis]
@@ -512,7 +544,6 @@ def test_1d_multioutput_lasso_and_multitask_lasso_cv():
     assert_almost_equal(clf.intercept_, clf1.intercept_[0])
 
 
-@pytest.mark.filterwarnings('ignore: You should specify a value')  # 0.22
 def test_sparse_input_dtype_enet_and_lassocv():
     X, y, _, _ = build_dataset(n_features=10)
     clf = ElasticNetCV(n_alphas=5)
@@ -530,7 +561,6 @@ def test_sparse_input_dtype_enet_and_lassocv():
     assert_almost_equal(clf.coef_, clf1.coef_, decimal=6)
 
 
-@pytest.mark.filterwarnings('ignore: You should specify a value')  # 0.22
 def test_precompute_invalid_argument():
     X, y, _, _ = build_dataset()
     for clf in [ElasticNetCV(precompute="invalid"),
@@ -549,25 +579,24 @@ def test_warm_start_convergence():
     n_iter_reference = model.n_iter_
 
     # This dataset is not trivial enough for the model to converge in one pass.
-    assert_greater(n_iter_reference, 2)
+    assert n_iter_reference > 2
 
     # Check that n_iter_ is invariant to multiple calls to fit
     # when warm_start=False, all else being equal.
     model.fit(X, y)
     n_iter_cold_start = model.n_iter_
-    assert_equal(n_iter_cold_start, n_iter_reference)
+    assert n_iter_cold_start == n_iter_reference
 
     # Fit the same model again, using a warm start: the optimizer just performs
     # a single pass before checking that it has already converged
     model.set_params(warm_start=True)
     model.fit(X, y)
     n_iter_warm_start = model.n_iter_
-    assert_equal(n_iter_warm_start, 1)
+    assert n_iter_warm_start == 1
 
 
 def test_warm_start_convergence_with_regularizer_decrement():
-    boston = load_boston()
-    X, y = boston.data, boston.target
+    X, y = load_boston(return_X_y=True)
 
     # Train a model to converge on a lightly regularized problem
     final_alpha = 1e-5
@@ -577,7 +606,7 @@ def test_warm_start_convergence_with_regularizer_decrement():
     # Fitting with high regularization is easier it should converge faster
     # in general.
     high_reg_model = ElasticNet(alpha=final_alpha * 10).fit(X, y)
-    assert_greater(low_reg_model.n_iter_, high_reg_model.n_iter_)
+    assert low_reg_model.n_iter_ > high_reg_model.n_iter_
 
     # Fit the solution to the original, less regularized version of the
     # problem but from the solution of the highly regularized variant of
@@ -586,7 +615,7 @@ def test_warm_start_convergence_with_regularizer_decrement():
     warm_low_reg_model = deepcopy(high_reg_model)
     warm_low_reg_model.set_params(warm_start=True, alpha=final_alpha)
     warm_low_reg_model.fit(X, y)
-    assert_greater(low_reg_model.n_iter_, warm_low_reg_model.n_iter_)
+    assert low_reg_model.n_iter_ > warm_low_reg_model.n_iter_
 
 
 def test_random_descent():
@@ -706,8 +735,7 @@ def test_enet_copy_X_False_check_input_False():
 def test_overrided_gram_matrix():
     X, y, _, _ = build_dataset(n_samples=20, n_features=10)
     Gram = X.T.dot(X)
-    clf = ElasticNet(selection='cyclic', tol=1e-8, precompute=Gram,
-                     fit_intercept=True)
+    clf = ElasticNet(selection='cyclic', tol=1e-8, precompute=Gram)
     assert_warns_message(UserWarning,
                          "Gram matrix was provided but X was centered"
                          " to fit intercept, "
@@ -750,7 +778,7 @@ def test_enet_float_precision():
                 coef[('simple', dtype)] = clf.coef_
                 intercept[('simple', dtype)] = clf.intercept_
 
-                assert_equal(clf.coef_.dtype, dtype)
+                assert clf.coef_.dtype == dtype
 
                 # test precompute Gram array
                 Gram = X.T.dot(X)
@@ -771,7 +799,7 @@ def test_enet_float_precision():
                 clf_multioutput.fit(X, multi_y)
                 coef[('multi', dtype)] = clf_multioutput.coef_
                 intercept[('multi', dtype)] = clf_multioutput.intercept_
-                assert_equal(clf.coef_.dtype, dtype)
+                assert clf.coef_.dtype == dtype
 
             for v in ['simple', 'multi']:
                 assert_array_almost_equal(coef[(v, np.float32)],
@@ -830,3 +858,188 @@ def test_warm_start_multitask_lasso():
     clf2 = MultiTaskLasso(alpha=0.1, max_iter=10)
     ignore_warnings(clf2.fit)(X, Y)
     assert_array_almost_equal(clf2.coef_, clf.coef_)
+
+
+@pytest.mark.parametrize('klass, n_classes, kwargs',
+                         [(Lasso, 1, dict(precompute=True)),
+                          (Lasso, 1, dict(precompute=False)),
+                          (MultiTaskLasso, 2, dict()),
+                          (MultiTaskLasso, 2, dict())])
+def test_enet_coordinate_descent(klass, n_classes, kwargs):
+    """Test that a warning is issued if model does not converge"""
+    clf = klass(max_iter=2, **kwargs)
+    n_samples = 5
+    n_features = 2
+    X = np.ones((n_samples, n_features)) * 1e50
+    y = np.ones((n_samples, n_classes))
+    if klass == Lasso:
+        y = y.ravel()
+    assert_warns(ConvergenceWarning, clf.fit, X, y)
+
+
+def test_convergence_warnings():
+    random_state = np.random.RandomState(0)
+    X = random_state.standard_normal((1000, 500))
+    y = random_state.standard_normal((1000, 3))
+
+    # check that the model fails to converge (a negative dual gap cannot occur)
+    with pytest.warns(ConvergenceWarning):
+        MultiTaskElasticNet(max_iter=1, tol=-1).fit(X, y)
+
+    # check that the model converges w/o warnings
+    with pytest.warns(None) as record:
+        MultiTaskElasticNet(max_iter=1000).fit(X, y)
+
+    assert not record.list
+
+
+def test_sparse_input_convergence_warning():
+    X, y, _, _ = build_dataset(n_samples=1000, n_features=500)
+
+    with pytest.warns(ConvergenceWarning):
+        ElasticNet(max_iter=1, tol=0).fit(
+            sparse.csr_matrix(X, dtype=np.float32), y)
+
+    # check that the model converges w/o warnings
+    with pytest.warns(None) as record:
+        Lasso(max_iter=1000).fit(sparse.csr_matrix(X, dtype=np.float32), y)
+
+    assert not record.list
+
+
+@pytest.mark.parametrize("precompute, inner_precompute", [
+    (True, True),
+    ('auto', False),
+    (False, False),
+])
+def test_lassoCV_does_not_set_precompute(monkeypatch, precompute,
+                                         inner_precompute):
+    X, y, _, _ = build_dataset()
+    calls = 0
+
+    class LassoMock(Lasso):
+        def fit(self, X, y):
+            super().fit(X, y)
+            nonlocal calls
+            calls += 1
+            assert self.precompute == inner_precompute
+
+    monkeypatch.setattr("sklearn.linear_model._coordinate_descent.Lasso",
+                        LassoMock)
+    clf = LassoCV(precompute=precompute)
+    clf.fit(X, y)
+    assert calls > 0
+
+
+def test_multi_task_lasso_cv_dtype():
+    n_samples, n_features = 10, 3
+    rng = np.random.RandomState(42)
+    X = rng.binomial(1, .5, size=(n_samples, n_features))
+    X = X.astype(int)  # make it explicit that X is int
+    y = X[:, [0, 0]].copy()
+    est = MultiTaskLassoCV(n_alphas=5, fit_intercept=True).fit(X, y)
+    assert_array_almost_equal(est.coef_, [[1, 0, 0]] * 2, decimal=3)
+
+
+@pytest.mark.parametrize('fit_intercept', [True, False])
+@pytest.mark.parametrize('alpha', [0.01])
+@pytest.mark.parametrize('normalize', [False, True])
+@pytest.mark.parametrize('precompute', [False, True])
+def test_enet_sample_weight_consistency(fit_intercept, alpha, normalize,
+                                        precompute):
+    """Test that the impact of sample_weight is consistent."""
+    rng = np.random.RandomState(0)
+    n_samples, n_features = 10, 5
+
+    X = rng.rand(n_samples, n_features)
+    y = rng.rand(n_samples)
+    params = dict(alpha=alpha, fit_intercept=fit_intercept,
+                  precompute=precompute, tol=1e-6, l1_ratio=0.5)
+
+    reg = ElasticNet(**params).fit(X, y)
+    coef = reg.coef_.copy()
+    if fit_intercept:
+        intercept = reg.intercept_
+
+    # sample_weight=np.ones(..) should be equivalent to sample_weight=None
+    sample_weight = np.ones_like(y)
+    reg.fit(X, y, sample_weight=sample_weight)
+    assert_allclose(reg.coef_, coef, rtol=1e-6)
+    if fit_intercept:
+        assert_allclose(reg.intercept_, intercept)
+
+    # sample_weight=None should be equivalent to sample_weight = number
+    sample_weight = 123.
+    reg.fit(X, y, sample_weight=sample_weight)
+    assert_allclose(reg.coef_, coef, rtol=1e-6)
+    if fit_intercept:
+        assert_allclose(reg.intercept_, intercept)
+
+    # scaling of sample_weight should have no effect, cf. np.average()
+    sample_weight = 2 * np.ones_like(y)
+    reg.fit(X, y, sample_weight=sample_weight)
+    assert_allclose(reg.coef_, coef, rtol=1e-6)
+    if fit_intercept:
+        assert_allclose(reg.intercept_, intercept)
+
+    # setting one element of sample_weight to 0 is equivalent to removing
+    # the corresponding sample
+    sample_weight = np.ones_like(y)
+    sample_weight[-1] = 0
+    reg.fit(X, y, sample_weight=sample_weight)
+    coef1 = reg.coef_.copy()
+    if fit_intercept:
+        intercept1 = reg.intercept_
+    reg.fit(X[:-1], y[:-1])
+    assert_allclose(reg.coef_, coef1, rtol=1e-6)
+    if fit_intercept:
+        assert_allclose(reg.intercept_, intercept1)
+
+    # check that multiplying sample_weight by 2 is equivalent
+    # to repeating corresponding samples twice
+    if sparse.issparse(X):
+        X = X.toarray()
+
+    X2 = np.concatenate([X, X[:n_samples//2]], axis=0)
+    y2 = np.concatenate([y, y[:n_samples//2]])
+    sample_weight_1 = np.ones(len(y))
+    sample_weight_1[:n_samples//2] = 2
+
+    reg1 = ElasticNet(**params).fit(
+            X, y, sample_weight=sample_weight_1
+    )
+
+    reg2 = ElasticNet(**params).fit(
+            X2, y2, sample_weight=None
+    )
+    assert_allclose(reg1.coef_, reg2.coef_)
+
+
+def test_enet_sample_weight_sparse():
+    reg = ElasticNet()
+    X = sparse.csc_matrix(np.zeros((3, 2)))
+    y = np.array([-1, 0, 1])
+    sw = np.array([1, 2, 3])
+    with pytest.raises(ValueError, match="Sample weights do not.*support "
+                                         "sparse matrices"):
+        reg.fit(X, y, sample_weight=sw, check_input=True)
+
+
+@pytest.mark.parametrize("backend", ["loky", "threading"])
+@pytest.mark.parametrize("estimator",
+                         [ElasticNetCV, MultiTaskElasticNetCV,
+                          LassoCV, MultiTaskLassoCV])
+def test_linear_models_cv_fit_for_all_backends(backend, estimator):
+    # LinearModelsCV.fit performs inplace operations on input data which is
+    # memmapped when using loky backend, causing an error due to unexpected
+    # behavior of fancy indexing of read-only memmaps (cf. numpy#14132).
+
+    if joblib.__version__ < LooseVersion('0.12') and backend == 'loky':
+        pytest.skip('loky backend does not exist in joblib <0.12')
+
+    # Create a problem sufficiently large to cause memmapping (1MB).
+    n_targets = 1 + (estimator in (MultiTaskElasticNetCV, MultiTaskLassoCV))
+    X, y = make_regression(20000, 10, n_targets=n_targets)
+
+    with joblib.parallel_backend(backend=backend):
+        estimator(n_jobs=2, cv=3).fit(X, y)
